/* 
 HIDUARTDlg.cpp : Implementation file for the 
 
 MOTO Development Group, Inc.
 Interface to the HIDUART for the Cypress CY7C63x01 USB Microcontroller
 A USB Human Interface Device Class Asynchronous Serial Transceiver
 Copyright (C) 2001  MOTO Development Group

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 Contacting MOTO Development Group:

 http://www.moto.com
 
 MOTO Development Group
 85 Second Street
 Suite 710
 San Francisco, CA 94105-3459
 
 415 281 4800 Tel
 415 281 3809 Fax

 marketing@moto.com
*/


#include "stdafx.h"
#include "HIDUART.h"
#include "HIDUARTDlg.h"
#include <process.h>

extern "C"{
#include "hidsdi.h"     // from Win98 DDK
#include "setupapi.h"   // from MS Platform SDK
}  

#define IDM_RCV_DATA    (WM_USER + 1)


/*
 These Vendor and Product ID values belong to MOTO Development Group.  If you use
 MOTO's vendor ID you may use PID 0100 only AND you may NOT use version 0100 or lower.
 If you want your own product ID MOTO may be willing to sell one of ours.  Otherwise, 
 USB.ORG is the sole broker of vendor IDs.

 Further, all of the above is true insofar as it is not in contravention of MOTO's
 obligations as a member of the USB Implementors Forum.
*/

// these constants must agree with those in firmware!
#define MOTO_USB_VID    (0x0C04)    // MOTO's USB Vendor ID
#define HIDUART_PID     (0x0100)    // MO1002 HID UART Product ID
#define HIDUART_VERSION (0x0100)    // This is MOTO's.  You must change this for your product!

#define RID_RECEIVE     (1)         // report ID associated with incoming data
#define RID_TRANSMIT    (2)         // report ID for transmitted data
#define RID_COMMAND     (3)         // command ID to set baud rate
#define MAX_DATA_LEN    (6)         // maximum transmitted data length
#define RCV_DATA_LEN    (8)         // 1 report ID, 1 count of valid bytes, and up to 6 data bytes

// available asynchronous serial data rates in MOTO's firmware
struct Rate {
    char *String;
    BYTE parameter;     // value passed to firmware to select given rate
} RateTable[] = {
        "default",  0,   // go to firmware's default, power-up rate
        "2400",     1,
        "4800",     2,
        "9600",     3,
        "19200",    4,
        "38400",    5,
        "57600",    6
};

#define NRATES (sizeof RateTable / sizeof RateTable[0])



// global flag read by received data thread and written by main thread
bool g_KeepGoing = false;

void __cdecl RcvThreadFunction(HANDLE hidHandle);

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
    CAboutDlg();

// Dialog Data
    //{{AFX_DATA(CAboutDlg)
    enum { IDD = IDD_ABOUTBOX };
    //}}AFX_DATA

    // ClassWizard generated virtual function overrides
    //{{AFX_VIRTUAL(CAboutDlg)
    protected:
    virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
    //}}AFX_VIRTUAL

// Implementation
protected:
    //{{AFX_MSG(CAboutDlg)
    //}}AFX_MSG
    DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
    //{{AFX_DATA_INIT(CAboutDlg)
    //}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CAboutDlg)
    //}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
    //{{AFX_MSG_MAP(CAboutDlg)
        // No message handlers
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHIDUARTDlg dialog

CHIDUARTDlg::CHIDUARTDlg(CWnd* pParent /*=NULL*/) :
CDialog(CHIDUARTDlg::IDD, pParent),
m_HidHandle(INVALID_HANDLE_VALUE),
m_MaxDataLen(MAX_DATA_LEN)
{
    //{{AFX_DATA_INIT(CHIDUARTDlg)
	//}}AFX_DATA_INIT
    // Note that LoadIcon does not require a subsequent DestroyIcon in Win32
    m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CHIDUARTDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CHIDUARTDlg)
	DDX_Control(pDX, IDC_BAUDRATE, m_BaudRate);
    DDX_Control(pDX, IDC_STATUS, m_Status);
    DDX_Control(pDX, IDC_SEND_DATA, m_SendData);
    DDX_Control(pDX, IDC_RCV_DATA, m_RcvDataCtrl);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CHIDUARTDlg, CDialog)
    //{{AFX_MSG_MAP(CHIDUARTDlg)
    ON_WM_SYSCOMMAND()
    ON_WM_PAINT()
    ON_WM_QUERYDRAGICON()
    ON_BN_CLICKED(IDC_OPEN, OnOpen)
    ON_BN_CLICKED(IDC_CLOSE, OnClose)
    ON_BN_CLICKED(IDC_SEND, OnSend)
    ON_MESSAGE(IDM_RCV_DATA, OnReceivedData)
    ON_EN_CHANGE(IDC_RCV_DATA, OnChangeRcvData)
	ON_CBN_SELCHANGE(IDC_BAUDRATE, OnSelchangeBaudrate)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CHIDUARTDlg message handlers


BOOL CHIDUARTDlg::OnInitDialog()
{
    CDialog::OnInitDialog();

    // Add "About..." menu item to system menu.

    // IDM_ABOUTBOX must be in the system command range.
    ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
    ASSERT(IDM_ABOUTBOX < 0xF000);

    CMenu* pSysMenu = GetSystemMenu(FALSE);
    if (pSysMenu != NULL)
    {
        CString strAboutMenu;
        strAboutMenu.LoadString(IDS_ABOUTBOX);
        if (!strAboutMenu.IsEmpty())
        {
            pSysMenu->AppendMenu(MF_SEPARATOR);
            pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
        }
    }

    // Set the icon for this dialog.  The framework does this automatically
    //  when the application's main window is not a dialog
    SetIcon(m_hIcon, TRUE);         // Set big icon
    SetIcon(m_hIcon, FALSE);        // Set small icon
    
    // init dropdown list
    char i;

    for (i= 0; i< NRATES; i++) {
        m_BaudRate.AddString(RateTable[i].String);
    }

    return TRUE;  // return TRUE  unless you set the focus to a control
}

void CHIDUARTDlg::OnSysCommand(UINT nID, LPARAM lParam)
{
    if ((nID & 0xFFF0) == IDM_ABOUTBOX)
    {
        CAboutDlg dlgAbout;
        dlgAbout.DoModal();
    }
    else
    {
        CDialog::OnSysCommand(nID, lParam);
    }
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CHIDUARTDlg::OnPaint() 
{
    if (IsIconic())
    {
        CPaintDC dc(this); // device context for painting

        SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);

        // Center icon in client rectangle
        int cxIcon = GetSystemMetrics(SM_CXICON);
        int cyIcon = GetSystemMetrics(SM_CYICON);
        CRect rect;
        GetClientRect(&rect);
        int x = (rect.Width() - cxIcon + 1) / 2;
        int y = (rect.Height() - cyIcon + 1) / 2;

        // Draw the icon
        dc.DrawIcon(x, y, m_hIcon);
    }
    else
    {
        CDialog::OnPaint();
    }
}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CHIDUARTDlg::OnQueryDragIcon()
{
    return (HCURSOR) m_hIcon;
}

//////////////////////////////////////////////////
// start of HIDUART specific functions
//////////////////////////////////////////////////

void CHIDUARTDlg::OnOpen() 
{
    if(INVALID_HANDLE_VALUE != m_HidHandle)
    {
        m_Status.SetWindowText("HIDUART is already open");
        return;
    }

    GUID hidGuid;

    HidD_GetHidGuid(&hidGuid);

    // open handle to the plug and play dev node for currently active HID devices
    HDEVINFO hDevInfo = SetupDiGetClassDevs(&hidGuid,
                                            NULL, // no enumerator 
                                            NULL, // no parent
                                            (DIGCF_PRESENT | DIGCF_INTERFACEDEVICE)); 
    if(!hDevInfo)
    {
        m_Status.SetWindowText("Couldn't get handle for HID devices");
        return;
    }

    // look for HID with our VID and PID
    SP_INTERFACE_DEVICE_DATA devInfoData;
    devInfoData.cbSize = sizeof (SP_INTERFACE_DEVICE_DATA);
    int deviceNo = 0;

    SetLastError(NO_ERROR);

    while(GetLastError() != ERROR_NO_MORE_ITEMS)
    {
        if(SetupDiEnumInterfaceDevice (hDevInfo,
                                       0, // don't care about specific PDOs
                                       &hidGuid,
                                       deviceNo,
                                       &devInfoData))
        {
            // get details about this device
            ULONG  requiredLength = 0;
            SetupDiGetInterfaceDeviceDetail(hDevInfo,
                                            &devInfoData,
                                            NULL, // no output buffer yet
                                            0,    // no output buffer yet
                                            &requiredLength,
                                            NULL); // not interested in the specific dev-node

            PSP_INTERFACE_DEVICE_DETAIL_DATA devDetail = (SP_INTERFACE_DEVICE_DETAIL_DATA*) malloc (requiredLength);
            devDetail->cbSize = sizeof(SP_INTERFACE_DEVICE_DETAIL_DATA);

            if(! SetupDiGetInterfaceDeviceDetail(hDevInfo,
                                                 &devInfoData,
                                                 devDetail,
                                                 requiredLength,
                                                 NULL,
                                                 NULL)) 
            {
                AfxMessageBox("Couldn't get HID details");
                free(devDetail);
                return;
            }

            // open a file to this device
            HANDLE hidHandle = CreateFile(devDetail->DevicePath,
                                          GENERIC_READ | GENERIC_WRITE,
                                          FILE_SHARE_READ | FILE_SHARE_WRITE,
                                          NULL, // no SECURITY_ATTRIBUTES structure
                                          OPEN_EXISTING, // No special create flags
                                          FILE_FLAG_OVERLAPPED, // to allow access to USB interrupt reports
                                          NULL); // No template file
            free(devDetail);

            if(INVALID_HANDLE_VALUE == hidHandle)
            {
                AfxMessageBox("Couldn't get HID handle");
                return;
            }

            // get the attributes of this device
            _HIDD_ATTRIBUTES hidAttributes;
            if(!HidD_GetAttributes(hidHandle, &hidAttributes))
            {
                AfxMessageBox("Couldn't get HID attributes");
                CloseHandle(hidHandle);
                return;
            }

            // just accept first device that matches our VID and PID
            if(MOTO_USB_VID == hidAttributes.VendorID &&
               HIDUART_PID  == hidAttributes.ProductID &&
               HIDUART_VERSION == hidAttributes.VersionNumber)

            {
                // show firmware version
                CString version;
                version.Format("HIDUART opened, firmware v%X", hidAttributes.VersionNumber);
                m_Status.SetWindowText(version);

                // save handle
                m_HidHandle = hidHandle;

                // max output data length (USED TO DEPEND) on firmware version
                m_MaxDataLen = MAX_DATA_LEN;
                
                // start thread for received data
                g_KeepGoing = true;
                if(_beginthread(RcvThreadFunction, 0, m_HidHandle) < 0)
                {
                    AfxMessageBox("Can't start receive data thread");
                }

                // all done
                break;
            }
            else
            {
                // try next HID
                CloseHandle(hidHandle);
                ++deviceNo;
            }
        }
    }

    if(INVALID_HANDLE_VALUE == m_HidHandle)
        m_Status.SetWindowText("Couldn't find HIDUART");

    SetupDiDestroyDeviceInfoList(hDevInfo);
}

void CHIDUARTDlg::OnClose() 
{
    if(INVALID_HANDLE_VALUE != m_HidHandle)
    {
        // tell receive data thread to end
        g_KeepGoing = false; 

        CloseHandle(m_HidHandle);

        m_HidHandle = INVALID_HANDLE_VALUE;

        m_Status.SetWindowText("HIDUART closed");
    }
    else
    {
        m_Status.SetWindowText("HIDUART not open");
    }
}

void CHIDUARTDlg::OnCancel() 
{
    // clean up before exiting
    OnClose();
    
    CDialog::OnCancel();
}

void CHIDUARTDlg::OnSend() 
{

    if(INVALID_HANDLE_VALUE == m_HidHandle)
    {
        m_Status.SetWindowText("HIDUART not open");
        return;
    }

    CString text;

    // read string from edit control
    m_SendData.GetWindowText(text);

    // add carriage return and line feed
    text += "\r\n";

    int len = text.GetLength();

    BYTE* reportBuf = new BYTE[m_MaxDataLen + 2];

    // report ID must be in first byte
    reportBuf[0] = RID_TRANSMIT;

    int offset = 0;

    while(len > 0)
    {
        // send up to m_MaxDataLen bytes at a time
        int thisLen = min(len, m_MaxDataLen);

        // actual data length must go in second byte
        reportBuf[1] = thisLen;

        // copy data to be sent into rest of buffer
        for(int i = 0; i < thisLen; i++)
            reportBuf[i + 2] = text[i + offset];

        // send this report to the device
        if(!HidD_SetFeature(m_HidHandle, reportBuf, m_MaxDataLen + 2))
        {
            AfxMessageBox("Couldn't send data!");
            delete reportBuf;
            return;
        }

        // get next chunk of up to m_MaxDataLen bytes
        len -= m_MaxDataLen;
        offset += m_MaxDataLen;
    }

    // clear sent data
//    m_SendData.SetWindowText("");

    delete reportBuf;
}

// thread function that receives data via USB interrupt input
void __cdecl RcvThreadFunction(HANDLE hidHandle)
{
    char rcvDataBuf[RCV_DATA_LEN];
    CString msg;

    // create Overlapped read structure and event
    HANDLE hIOWaiter = CreateEvent(NULL, TRUE, FALSE, NULL);

    if(NULL == hIOWaiter)
        goto exit2;

    OVERLAPPED ol;
    ol.Offset = 0;
    ol.OffsetHigh = 0;
    ol.hEvent = hIOWaiter;

    // loop waiting for events, till g_KeepGoing goes false
    for(;;)
    {
        // initiate overlapped read
        DWORD rcvdBytes;
        ResetEvent(hIOWaiter);
        if(!ReadFile(hidHandle, rcvDataBuf, RCV_DATA_LEN, &rcvdBytes, &ol))
        {
            // Check for read errors
            DWORD err = GetLastError();
            if(err != ERROR_IO_PENDING && err != ERROR_DEVICE_NOT_CONNECTED && g_KeepGoing)
            {
                msg.Format("Read error: %d", err);
                AfxMessageBox(msg);
                goto exit1;
            }


            // wait for read to complete (check for g_KeepGoing going false every 100ms)
            while(WaitForSingleObject(hIOWaiter, 100) == WAIT_TIMEOUT)
            {
                if(!g_KeepGoing)
                {
                    if(INVALID_HANDLE_VALUE != hidHandle) // if handle hasn't already been closed
                    {
                        // cancel the pending read
                        CancelIo(hidHandle);
                    }
                    goto exit1;
                }
            }

            // get read result, i.e. bytes transferred
            if(!GetOverlappedResult(hidHandle, &ol, &rcvdBytes, false))
            {
                err = GetLastError();
                if(err != ERROR_DEVICE_NOT_CONNECTED && g_KeepGoing)
                {
                    msg.Format("GetOverlappedResult error: %d", err);
                    AfxMessageBox(msg);
                }

                continue;
            }
        }

        // make sure there's something there
        if(rcvdBytes < RCV_DATA_LEN)
        {
            if(g_KeepGoing)
            {
                msg.Format("Only read %d bytes", rcvdBytes); 
                AfxMessageBox(msg);
            }

            continue;
        }

        // notify app of new data
        // (note rcvDataBuf[0] just contains the report ID)
        AfxGetMainWnd()->SendMessage(IDM_RCV_DATA, 0, (LPARAM)(rcvDataBuf + 1));
    }

exit1:
    CloseHandle(hIOWaiter);

exit2:
    _endthread();
}

// handler for IDM_RCV_DATA messages
// lParam points to the data buffer, the first element of which tells the number of valid bytes 
LONG CHIDUARTDlg::OnReceivedData(UINT wParam, LONG lParam)
{
    // this routine may be too slow to handle data coming in at rates above 4800 baud
    // it also never empties the received data string m_RcvData, and so will get even 
    // slower as that string grows

    if(!g_KeepGoing)        // thread has been killed, so buffer may no longer exist
        return (LRESULT) 0;

    int len = *(BYTE*)lParam;
    char* buf = (char*)(lParam + 1);

    // zero terminate the returned string
    buf[len] = '\0';

    // add to existing received data
    m_RcvData += buf;

    // add LF to terminal CR
    if(buf[len - 1] == '\r')
        m_RcvData += "\n";

    // display it
    m_RcvDataCtrl.SetWindowText(m_RcvData);

    // scroll to bottom  
    m_RcvDataCtrl.LineScroll(m_RcvDataCtrl.GetLineCount());

    return (LRESULT) 0;
}

void CHIDUARTDlg::OnChangeRcvData() 
{
    // user must have changed edit control contents,
    // so update our received data string
    m_RcvDataCtrl.GetWindowText(m_RcvData);
}

void CHIDUARTDlg::OnSelchangeBaudrate() 
{
	// TODO: Add your control notification handler code here
    if(INVALID_HANDLE_VALUE == m_HidHandle)
    {
        m_Status.SetWindowText("HIDUART not open");
        return;
    }

    BYTE* reportBuf = new BYTE[3];

    // report ID must be in first byte
    reportBuf[0] = RID_COMMAND;

    int rateindex;

    rateindex = m_BaudRate.GetCurSel();

    // baud command byte in position 2
    reportBuf[1] = RateTable[rateindex].parameter;

    // send this report to the device
    if(!HidD_SetFeature(m_HidHandle, reportBuf, m_MaxDataLen + 2))
    {
        m_Status.SetWindowText("Unable to transmit!");
    } else {
        m_Status.SetWindowText("Rate Set!");
    }
    delete reportBuf;

    
}
