;=======================================================================================================
; RAM.INC
;
; MOTO Development Group, Inc.
; HIDUART for the Cypress CY7C63x01 USB Microcontroller.  This code is
; based on Cypress Semiconductor's firmware frameworks code.
; A USB Human Interface Device Class Asynchronous Serial Transceiver
; Copyright (C) 2001  MOTO Development Group
;
; This program is free software; you can redistribute it and/or
; modify it.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
;
; Contacting MOTO Development Group:
;
; http://www.moto.com
; 
; MOTO Development Group
; 85 Second Street
; Suite 710
; San Francisco, CA 94105-3459
; 
; 415 281 4800 Tel
; 415 281 3809 Fax
;
; marketing@moto.com
;
;=======================================================================================================
;   
;   Ram Variable Declarations and Macros
;   
;========================================

; These Vendor and Product ID values belong to MOTO Development Group.  If you use
; MOTO's vendor ID you may use PID 0100 only AND you may NOT use version 0100 or lower.
; If you want your own product ID MOTO may be willing to sell one of ours.  Otherwise, 
; USB.ORG is the sole broker of vendor IDs.
;
; Further, all of the above is true only insofar as it is not in contravention of MOTO's
; obligations as a member of the USB Implementors Forum.
;

MOTO_VID:                   equ 0C04h
MOTO_PID:                   equ 0100h
MOTO_VER:                   equ 0100h       ; USB Firmware Version No. (BCD)


bottom_of_ram:          equ 00h
; program stack area.  experiments under full stress indicate
; that no more than 0Ch bytes are used.  So we'll be conservative
; and start the ram area at 18h, giving about 10 bytes (83%) of headroom
; over the worst case seen

protocol_status:        equ 18h
remote_wakeup_status:   equ 19h
suspend_counter:        equ 1Ah
interrupt_mask:         equ 1Bh
loop_counter:           equ 1Ch
endp0_data_toggle:      equ 1Dh
data_count:             equ 1Eh
data_start:             equ 1Fh
configuration_status:   equ 20h

;APPLICATION SPECIFIC RAM VARIABLES
serial_rx_reg:          equ 21h     ; these aren't used at the same time, so they 
serial_tx_reg:          equ 21h     ; can be at the same loc
ep1_byte_count:         equ 22h     ;track the number of bytes we're putting in ep1

rx_inptr:               equ 23h     ;Points to the next available space in the serial buffer
rx_outptr:              equ 24h     ;Points to the next data to be read in the serial buffer
current_reportid:       equ 25h     ;Holds the REPORTID of the current Report.  This variable
                                    ;is set during a GetReport or a SetReport.
baud_bitdelay:          equ 26h     ;Adjust this delay for various baud rates.
serial_buf_datacount:   equ 27h     ;Holds the number of valid data bytes contained in rx buffer
fwcr:                   equ 28h     ;FirmWare Control Register.  general purpose flags.
transmit_count:         equ 29h     ;the number of bytes in a setreport that we actually want to tx

RX_BUF_PTR_START:       equ 2Bh
RX_BUF_PTR_END:         equ 67h      
SERIAL_BUF_SIZE:        equ (RX_BUF_PTR_END - RX_BUF_PTR_START)


; examination of stack usage under full stress indicates only 
; four bytes used.  We'll allow for eight.  RX_BUF can go all
; the way to 67.

DATA_STACK_POINTER_START:   equ 70h

;========================================
;   
;   FIFO Declarations
;   
;========================================

EP0_FIFO:                   equ  70h            ; control endpoint 0 fifo
EP1_FIFO:                   equ  78h            ; data endpoint 1 fifo

bmRequestType:          equ  EP0_FIFO
bRequest:               equ  EP0_FIFO + 1
wValue:                 equ  EP0_FIFO + 2       ; default wValue (8-bits)
wValueHi:               equ  EP0_FIFO + 3
wIndex:                 equ  EP0_FIFO + 4       ; default wIndex (8-bits)
wIndexHi:               equ  EP0_FIFO + 5 
wLength:                equ  EP0_FIFO + 6       ; default wLength (8-bits)
wLengthHi:              equ  EP0_FIFO + 7

top_of_ram:         equ 7fh



;========================================
;   
;   Constant Declarations
;   
;========================================

EP1_REPORT_LENGTH:  equ 07h

;Application specific gier masks                    
ENUMERATE_MASK:     equ  EP0IE_B 
ENUMERATED_MASK:    equ  EP0IE_B+GPIOIE_B+EP1IE_B
TRANSMITTING_MASK:  equ  EP0IE_B                        ; used during uart transmits from SETUP thru end of the OUT
SUSPEND_MASK:       equ  00h                  
INTDISABLE_MASK:    equ  00h

;========================================
;   
;   USB Constant Declarations
;   
;========================================

; values for bmRequestType

HOST_TO_DEVICE:         equ 00h
HOST_TO_INTERFACE:      equ 01h
HOST_TO_ENDPOINT:       equ 02h

DEVICE_TO_HOST:         equ 80h
INTERFACE_TO_HOST:      equ 81h
ENDPOINT_TO_HOST:       equ 82h

CLASS:                  equ 20h
VENDOR:                 equ 40h

; other EP0 constants

EP1_STALLED:            equ  0      ; recipient endpoint
REMOTE_WAKEUP:          equ  1      ; recipient device

DISABLE_REMOTE_WAKEUP:  equ  0         ; bit[1] = 0
ENABLE_REMOTE_WAKEUP:   equ  2         ; bit[1] = 1

SUSPEND_TIME:           equ 15h     ; found empirically on 3650 ICE

;========================================
;   
;   Application Specific aliases
;   
;========================================

P0PULLUPS:                  equ 00h     ;P0.*, Enable pullups on all

; Port 0
SERIAL0TXBIT:               equ 01h     ;P0_data
SERIAL0RXBIT:               equ 02h     ;P0_data

; Port 1
nCTS:                       equ 01h     ;BUSY/nCTS: goes high when we can't listen
nDATA_AVAILABLE:            equ 02h     ;goes low when we have something to tx (parallel mode only)
nOE:                        equ 04h     ;enable 1 byte output (parallel mode only)
nRST:                       equ 08h     ;the reset output line

XON:                        equ 11h     ;ascii value for XON
XOFF:                       equ 13h     ;ascii value for XOFF

; FWCR general purpose flag bits
TIMER_SYNC:                 equ 01h     ; crude timer overflow sense bit
FIRSTBYTE_FLG_MASK:         equ 02h     ;bit set to indicate we're expecting report OUT tokens
BUSY_TRANSMITTING:          equ 04h     ;set during SETUP-OUT-OUT-OUT-...-IN exchange.  It was found
                                        ; that the background (main) task suspendcounter sees the timer
                                        ; ticks such a way that it incorrectly guesses that it has detected
                                        ; a lack of usb activity.  we use this flag to indicate to the background
                                        ; that this foreground is hogging cycles-- so don't look for usb suspension
FLOW_CTL_OFF:               equ 08h     ; set when our rx buffer is full

RID_COMMAND:                equ 3h      ; reserved for future use.  we might want to change baud rates with this
RID_TRANSMIT:               equ 2h      ; this set report ID is used when host wants to tx bytes out uart
RID_RECEIVE:                equ 1h      ; data streamed up ep1 from uart rx

; timing at a 2us granularity for each bit (not including overhead for bit routine)
BAUD_BITDELAY_57600:        equ 05          ;this works, but it might be marginal on some systems.
BAUD_BITDELAY_38400:        equ 09          ;this works pretty well
BAUD_BITDELAY_19200:        equ 22          ;better and better
BAUD_BITDELAY_9600:         equ 48          ;good
BAUD_BITDELAY_4800:         equ 100         ;good
BAUD_BITDELAY_2400:         equ 205         ;spot-on

BAUD_BITDELAY_DEFAULT:      equ BAUD_BITDELAY_9600

;*********************************************************
; MACROS

MACRO debug1on
        iord P0_data
        and a, 0fbh
        or a, SERIAL0RXBIT              ; ensure pulled up
        iowr P0_data
ENDM


MACRO debug1off
        iord P0_data
        or a, (4+SERIAL0RXBIT)              ; ensure pulled up
        iowr P0_data
ENDM


MACRO debug2on
        iord P0_data                    ; +5
        and a, 0f7h                     ; +4
        or a, SERIAL0RXBIT              ; +4 ensure pulled up
        iowr P0_data                    ; +5
ENDM


MACRO debug2off
        iord P0_data
        or a, (8+SERIAL0RXBIT)              ; ensure pulled up
        iowr P0_data
ENDM

MACRO debug2toggle
        iord P0_data
        or a, SERIAL0RXBIT
        xor a, 8              ; ensure pulled up
        iowr P0_data
ENDM


MACRO   BIT_DELAY
;======================================================================
;	
; bitdelay
;
;   DESCRIPTION:    adjustable bit delay
;                  
;   USAGE:          BIT_DELAY
;         
; each count in the loop takes +24 cycles = 2us
;
;
;=======================================================================
    cmp a, 0                    ;+5  instruction inserted only for timing.
                                ; re-create the 5 cycle overhead for this function
bdloop:
    nop                         ; +4 
    nop                         ; +4 
    cmp a, [0]                  ; +7 inserted only for timing

    dec a                       ; +4
    jnz bdloop                  ; +5

ENDM





MACRO DELAY10MS
;============================
;   Delay10ms - approximately
; we don't have XPAGE protection
; around this so it may go longer.
;============================

    mov x,051                           ;Preload counters
    mov a,FFh

.Delay10ms_msB:
	iowr wdt                            ; kick the dog
.Delay10ms_lsB:
    dec A
    jnc .Delay10ms_lsB
    dec X
    jnc .Delay10ms_msB


ENDM


MACRO FLOW_OFF
    iord P1_data
    or a,nCTS                               ; set nCTS high (i.e., not clear to send)
    iowr P1_data       
    
    mov a, FLOW_CTL_OFF
    or [fwcr], a 
ENDM


MACRO FLOW_ON
    iord P1_data
    and a,~nCTS                             ; set nCTS low (i.e., clear to send)
    iowr P1_data        

    mov a, ~FLOW_CTL_OFF
    and [fwcr], a 
ENDM
