;=======================================================================================================
; USB1REGS.INC
;
; MOTO Development Group, Inc.
; HIDUART for the Cypress CY7C63x01 USB Microcontroller.  This code is
; based on Cypress Semiconductor's firmware frameworks code.
; A USB Human Interface Device Class Asynchronous Serial Transceiver
; Copyright (C) 2001  MOTO Development Group
;
; This program is free software; you can redistribute it and/or
; modify it.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  
;
; Contacting MOTO Development Group:
;
; http://www.moto.com
; 
; MOTO Development Group
; 85 Second Street
; Suite 710
; San Francisco, CA 94105-3459
; 
; 415 281 4800 Tel
; 415 281 3809 Fax
;
; marketing@moto.com
;
;=======================================================================================================
;	usb1regs.h
;
;	This file contains the SFR regsiter declarations
;	CY7C6300x, 6310x and 6360x
;
;	
;========================================
;	
;	General Constant Declarations
;	
;========================================

BIT0:			equ	01h
BIT1:			equ	02h
BIT2:			equ	04h
BIT3:			equ	08h
BIT4:			equ	10h
BIT5:			equ	20h
BIT6:			equ	40h
BIT7:			equ	80h

;========================================
;	
;	SFR Declarations
;	
;========================================

; I/O ports

P0_data:            	equ   00h      ; GPIO data port 0
	P0.0:			equ	BIT0
	P0.1:			equ	BIT1
	P0.2:			equ	BIT2
	P0.3:			equ	BIT3
	P0.4:			equ	BIT4
	P0.5:			equ	BIT5
	P0.6:			equ	BIT6
	P0.7:			equ	BIT7

P1_data:            	equ   01h      ; GPIO data port 1
	P1.0:			equ	BIT0
	P1.1:			equ	BIT1
	P1.2:			equ	BIT2
	P1.3:			equ	BIT3
	P1.4:			equ	BIT4
	P1.5:			equ	BIT5
	P1.6:			equ	BIT6
	P1.7:			equ	BIT7

P0_interrupt:       	equ   04h      ; Interrupt enable for port 0
	IE0.0:		equ	BIT0
	IE0.1:		equ	BIT1
	IE0.2:		equ	BIT2
	IE0.3:		equ	BIT3
	IE0.4:		equ	BIT4
	IE0.5:		equ	BIT5
	IE0.6:		equ	BIT6
	IE0.7:		equ	BIT7
	
P1_interrupt:       	equ   05h      ; Interrupt enable for port 1
	IE1.0:		equ	BIT0
	IE1.1:		equ	BIT1
	IE1.2:		equ	BIT2
	IE1.3:		equ	BIT3
	IE1.4:		equ	BIT4
	IE1.5:		equ	BIT5
	IE1.6:		equ	BIT6
	IE1.7:		equ	BIT7

P0_pullup:          	equ  	08h      ; Pullup resistor control for port 0
	PULL0.0:		equ	BIT0
	PULL0.1:		equ	BIT1
	PULL0.2:		equ	BIT2
	PULL0.3:		equ	BIT3
	PULL0.4:		equ	BIT4
	PULL0.5:		equ	BIT5
	PULL0.6:		equ	BIT6
	PULL0.7:		equ	BIT7
	
P1_pullup:         	equ   09h      ; Pullup resistor control for port 1
	PULL1.0:		equ	BIT0
	PULL1.1:		equ	BIT1
	PULL1.2:		equ	BIT2
	PULL1.3:		equ	BIT3
	PULL1.4:		equ	BIT4
	PULL1.5:		equ	BIT5
	PULL1.6:		equ	BIT6
	PULL1.7:		equ	BIT7

; USB ports

EP0_TX_Config:    	equ   10h     ; EP0 transmit configuration
	EP0_TX_COUNT:	equ	0fh
	ERR_B:		    equ	BIT4
	STALL_B:		equ	BIT5
	DATA10_B:		equ	BIT6
	INEN_B:		    equ	BIT7

EP1_TX_Config:    	equ   11h     ; EP1 transmit configuration
	EP1_TX_COUNT:	equ	0fh
	EP1EN_B:		equ	BIT4
;	STALL_B:		equ	BIT5		;already declared for EP0
;	DATA10_B:		equ	BIT6		;already declared for EP0
;	INEN_B:		    equ	BIT7		;already declared for EP0

USB_DA:			    equ   12h     ; USB device address assigned by host
	ADR0_B:		    equ	BIT0
	ADR1_B:		    equ	BIT1
	ADR2_B:		    equ	BIT2
	ADR3_B:		    equ	BIT3
	ADR4_B:		    equ	BIT4
	ADR5_B:		    equ	BIT5
	ADR6_B:		    equ	BIT6

USB_SCR:     		equ   13h     ; USB status and control register
	BUSACT_B:		equ	BIT0
	FORCEK_B:		equ	BIT1
	FORCEJ_B:		equ	BIT2
	STATOUTS_B:		equ	BIT3
	ENOUTS_B:		equ	BIT4

EP0_RX_Status:    	equ   14h     ; USB EP0 receive status
	SETUP_B:		equ	BIT0
	OUT_B:			equ	BIT1
	IN_B:			equ	BIT2
	TOGGLE_B:		equ	BIT3
	EP0_RX_COUNT:	equ	f0h

; control ports

gier:		       	equ   20h     ; Global interrupt enable
	128IE_B:		equ	BIT1
	1024IE_B:		equ	BIT2
	EP0IE_B:		equ	BIT3
	EP1IE_B:		equ	BIT4
	GPIOIE_B:		equ	BIT6
	CEXTIE_B:		equ	BIT7

wdt:               	equ   21h     ; clear watchdog Timer
timer:                  equ   23h     ; free-running Timer
cext:				equ			22h		;Address of the Cext register.

; GPIO Isink registers

P0_sink0:           	equ   30h	;Port 0 Bit 0
P0_sink1:           	equ   31h
P0_sink2:           	equ   32h
P0_sink3:           	equ   33h
P0_sink4:           	equ   34h
P0_sink5:           	equ   35h
P0_sink6:           	equ   36h
P0_sink7:           	equ   37h	;Port 0 Bit 7

P1_sink0:           	equ   38h	;Port 1 Bit 0
P1_sink1:           	equ   39h
P1_sink2:           	equ   3Ah
P1_sink3:           	equ   3Bh	
P1_sink4:           	equ   3Ch
P1_sink5:           	equ   3Dh
P1_sink6:           	equ   3Eh
P1_sink7:           	equ   3Fh	;Port 0 Bit 7

; control port

scr:					equ	FFh	;Staus and Control Register
RUN_B:					equ	BIT0
SUSPEND_B:				equ	BIT3
POR_B:					equ	BIT4
USBR_B:					equ	BIT5
WDR_B:					equ	BIT6
RESET_MASK:				equ	70h



